# The IP address of your phone; You can hot-reload how many phones you want.
# use "127.0.0.1" if you are using an emulator.
PHONE_IPS = ["192.168.1.68"]

# If True, it's going to send the app to smartphone every time you press Ctrl+S
HOT_RELOAD_ON_PHONE = False

# You can stream using "USB" or "WIFI"
# If your USB cable is connected, use "USB", otherwise use "WIFI"
# To use WIFI, you need connect your phone to the same network as your computer
STREAM_USING = "USB"

# The port where the server will be listening
# It can be any port
PORT = 5555

# Add the python files you want to watch here, i.e., if they change, the app will be reloaded
WATCHED_FILES = ["main.py"]

# Add the folders you want to watch recursively here, i.e., if **ANY FILE** inside them changes, the app will be reloaded
WATCHED_FOLDERS_RECURSIVELY = []

# Add the folders where you have your .kv files here
# If you don't put them here, the .kv files won't be reloaded
# this is recursive, i.e., it will watch all folders inside the folders you put here
WATCHED_KV_FOLDERS_RECURSIVELY = []

# Watched folders but not recursively, i.e., **ANY FILE** inside the first level of the folder
WATCHED_FOLDERS = []

# Add the folders where you have your .kv files here
# If you don't put them here, the .kv files won't be reloaded
# this is not recursive, i.e., it will only watch the first level of the folders you put here
WATCHED_KV_FOLDERS = []

# If you want to logcat services, put here the name of each service
# For example, ["MyService", "MyOtherService"]
SERVICE_NAMES = []

# If you want to hotreload your services, put the name of the service files here
# For example, ["service.py", "service2.py"]
SERVICE_FILES = []

# If you want to exclude some files or folders, from being copied to the phone, add them here
FOLDERS_AND_FILES_TO_EXCLUDE_FROM_PHONE = [
    "*.pyc",
    "__pycache__",
    ".buildozer",
    ".venv",
    ".vscode",
    ".git",
    ".pytest_cache",
    ".DS_Store",
    ".env",
    ".dmypy.json",
    ".mypy_cache/",
    ".dmypy.json",
    "dmypy.json",
    "env/",
    "ENV/",
    "env.bak/",
    "venv/",
    "venv.bak/",
    "bin",
    "buildozer.spec",
    "poetry.lock",
    "pyproject.toml",
    "temp",
    "tests",
    "app_copy.zip",
    "send_app_to_phone.py",
    ".gitignore",
    "README.md",
]

# The X position of the window generated by scrcpy
WINDOW_X = "1200"

# The Y position of the window generated by scrcpy
WINDOW_Y = "100"

# The width of the window generated by scrcpy
WINDOW_WIDTH = "280"

# The title of the window generated by scrcpy
WINDOW_TITLE = "Kivy Reloader"

# If you want to show the touches on the screen, set this to True
SHOW_TOUCHES = False

# If you want the screen to be always on, set this to True
STAY_AWAKE = False

# If you want to turn the screen off, set this to True
TURN_SCREEN_OFF = False

# If you want the window to be always on top, set this to True
ALWAYS_ON_TOP = True

assert (
    len(WATCHED_FILES) + len(WATCHED_FOLDERS) + len(WATCHED_FOLDERS_RECURSIVELY) > 0
), "No files or folders to watch"
